@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 # plot.py
-"""Plotting for DVHA-Stats."""
+"""Basic plotting class objects for DVHA-Stats based on matplotlib"""
 #
 # Copyright (c) 2020 Dan Cutright
 # This file is part of DVHA-Stats, released under a MIT license.
@@ -17,13 +17,23 @@ FIGURE_COUNT = 1
 
 
 def get_new_figure_num():
+    """Get a number for a new matplotlib figure
+
+    Returns
+    ----------
+    int
+        Figure number
+    """
     global FIGURE_COUNT
     FIGURE_COUNT += 1
     return FIGURE_COUNT - 1
 
 
 class Chart:
+    """Base class for charts"""
+
     def __init__(self, title=None):
+        """Initialization of Chart base class"""
         self.title = title
         self.figure = plt.figure(get_new_figure_num())
 
@@ -31,13 +41,16 @@ class Chart:
             self.figure.suptitle(title, fontsize=16)
 
     def show(self):
+        """Display this figure"""
         self.activate()
         plt.show()
 
     def activate(self):
+        """Activate this figure"""
         plt.figure(self.figure.number)
 
     def close(self):
+        """Close this figure"""
         plt.close(self.figure.number)
 
 
@@ -90,8 +103,8 @@ class Plot(Chart):
             Specify the scatter plot circle color
         """
         Chart.__init__(self, title=title)
-        self.x = np.linspace(1, len(y), len(y)) if x is None else x
-        self.y = y
+        self.x = np.linspace(1, len(y), len(y)) if x is None else np.array(x)
+        self.y = np.array(y) if not isinstance(y, np.ndarray) else y
         self.show = show
         self.xlabel = xlabel
         self.ylabel = ylabel
@@ -111,20 +124,24 @@ class Plot(Chart):
             plt.show()
 
     def __add_labels(self):
+        """Set the x and y axes labels to figure"""
         plt.xlabel(self.xlabel)
         plt.ylabel(self.ylabel)
 
     def __add_data(self):
+        """Add scatter and/or line data to figure"""
         if self.scatter:
             self.add_scatter()
         if self.line:
             self.add_default_line()
 
     def add_scatter(self):
+        """Add scatter data to figure"""
         self.activate()
         plt.scatter(self.x, self.y, color=self.scatter_color)
 
     def add_default_line(self):
+        """Add line data to figure"""
         self.activate()
         plt.plot(
             self.x,
@@ -151,7 +168,6 @@ class Plot(Chart):
             Specify the line width
         line_style : str
             Specify the line style
-        :
         """
         self.activate()
         plt.plot(
@@ -164,7 +180,7 @@ class Plot(Chart):
 
 
 class ControlChart(Plot):
-    """ControlChart class object inherited from Plot"""
+    """ControlChart class object"""
 
     def __init__(
         self,
@@ -385,7 +401,18 @@ class PCAFeatureMap(HeatMap):
 
     @staticmethod
     def get_ordinal(n):
-        """Convert 1, 2, 3, 4, etc. to 1st, 2nd, 3rd, 4th, etc."""
+        """Convert number to its ordinal (e.g., 1 to 1st)
+
+        Parameters
+        ----------
+        n : int
+            Number to be converted to ordinal
+
+        Returns
+        ----------
+        str
+            the ordinal of n
+        """
         return "%d%s" % (
             n,
             "tsnrhtdd"[(n // 10 % 10 != 1) * (n % 10 < 4) * n % 10 :: 4],
