@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 # stats.py
-"""Statistical calculations for DVH Analytics"""
+"""Statistical calculations and class objects"""
 #
 # Copyright (c) 2020 Dan Cutright
 # Copyright (c) 2020 Arka Roy
@@ -11,438 +11,69 @@
 #    available at https://github.com/cutright/DVHA-Stats
 
 
-from os.path import isfile, splitext
 import numpy as np
-from scipy.stats import beta
-from dvhastats.utilities import dict_to_array, csv_to_dict, moving_avg
-from dvhastats import plot
 from scipy import stats as scipy_stats
 from sklearn import linear_model
 from sklearn.metrics import mean_squared_error, r2_score
-from sklearn.decomposition import PCA as sklearnPCA
+from sklearn.decomposition import PCA as sklearn_PCA
 from regressors import stats as regressors_stats
 
 
-class DVHAStatsBaseClass:
-    """Base Class for DVHAStats objects and child objects"""
-
-    def __init__(self):
-        """Initialization of DVHAStatsBaseClass for common attr/methods"""
-        self.plots = []
-
-    def close(self, figure_number):
-        """Close a plot by figure_number"""
-        for i, p in enumerate(self.plots):
-            if p.figure.number == figure_number:
-                p.close()
-                self.plots.pop(i)
-                return
-
-
-class DVHAStats(DVHAStatsBaseClass):
-    def __init__(
-        self,
-        data,
-        var_names=None,
-        x_axis=None,
-        avg_len=5,
-        del_const_vars=False,
-    ):
-        """Class used to calculated various statistics
-
-        Parameters
-        ----------
-        data : numpy.array, dict, str
-            Input data (2-D) with N rows of observations and
-            p columns of variables.  The CSV file must have a header row
-            for column names.
-        var_names : list of str, optional
-            If data is a numpy array, optionally provide the column names.
-        x_axis : numpy.array, list, optional
-            Specify x_axis for plotting purposes. Default is based on row
-            number in data
-        avg_len : int
-            When plotting raw data, a trend line will be plotted using this
-            value as an averaging length. If N < avg_len + 1 will not
-            plot a trend line
-        del_const_vars : bool
-            Automatically delete any variables that have constant data. The
-            names of these variables are stored in the excluded_vars attr.
-            Default value is False.
+class Histogram:
+    def __init__(self, y, bins, nan_policy="omit"):
         """
-        DVHAStatsBaseClass.__init__(self)
-        if isinstance(data, np.ndarray):
-            self.data = data
-            self.var_names = (
-                var_names
-                if var_names is not None
-                else list(range(data.shape[1]))
-            )
-        elif isinstance(data, dict):
-            data = dict_to_array(data)
-            self.data = data["data"]
-            self.var_names = data["var_names"]
-        elif isfile(data) and splitext(data)[1] == ".csv":
-            data = dict_to_array(csv_to_dict(data, dtype=float))
-            self.data = data["data"]
-            self.var_names = data["var_names"]
-        else:
-            msg = (
-                "Invalid data provided - "
-                "must be a numpy array, dict, or .csv file"
-            )
-            raise NotImplementedError(msg)
 
-        self.x_axis = x_axis
 
-        self.deleted_vars = []
+        y: array-like
+            Input array.
+        nan_policy : str
+            Value must be one of the following: ‘propagate’, ‘raise’, ‘omit’
+            Defines how to handle when input contains nan. The following
+            options are available (default is ‘omit’):
+            ‘propagate’: returns nan
+            ‘raise’: throws an error
+            ‘omit’: performs the calculations ignoring nan values
+        """
 
-        self.box_cox_data = None
-
-        self.avg_len = avg_len
-
-        if del_const_vars:
-            self.del_const_vars()
-
-    def get_data_by_var_name(self, var_name):
-        """Get the single variable array based on var_name"""
-        index = self.get_index_by_var_name(var_name)
-        return self.data[:, index]
-
-    def get_index_by_var_name(self, var_name):
-        """Get the variable index by var_name"""
-        if var_name in self.var_names:
-            index = self.var_names.index(var_name)
-        elif isinstance(var_name, int) and var_name in range(
-            self.variable_count
-        ):
-            return var_name
-        else:
-            msg = "%s is not a valid var_name\n%s" % (
-                var_name,
-                ",".join(self.var_names),
-            )
-            raise AttributeError(msg)
-        return index
+        self.y = process_nan_policy(y, nan_policy)
+        self.bins = bins
+        self.nan_policy = nan_policy
 
     @property
-    def observations(self):
-        """Number of observations"""
-        return self.data.shape[0]
+    def mean(self):
+        """The mean value of the input array"""
+        return np.mean(self.y)
 
     @property
-    def variable_count(self):
-        """Number of variables"""
-        return self.data.shape[1]
+    def median(self):
+        """The median value of the input array"""
+        return np.median(self.y)
 
-    def del_const_vars(self):
-        """Permanently remove constant variables"""
-        self.deleted_vars = self.constant_vars
-        self.data = self.non_const_data
-
-    def correlation_matrix(self, corr_type="Pearson"):
-        """Get a Pearson-R and Spearman correlation matrices
-
-        Parameters
-        ----------
-        corr_type : str
-            Either "Pearson" or "Spearman"
-
-        Returns
-        ----------
-        CorrelationMatrix
-            A CorrelationMatrix class object
-        """
-        return CorrelationMatrix(
-            self.data, self.var_names, corr_type=corr_type
-        )
+    @property
+    def std(self):
+        """The standard deviation of the input array"""
+        return np.std(self.y)
 
     @property
     def normality(self):
-        """Calculate the normality and associated p-values for each variable
-
-        Returns
-        ----------
-        np.ndarray, np.ndarray
-            A tuple of 1-D arrays are returned: Normality and its
-            p-values.
-        """
-        norm, p = np.zeros(self.variable_count), np.zeros(self.variable_count)
-        for i in range(self.variable_count):
-            norm[i], p[i] = scipy_stats.normaltest(
-                self.data[:, i], nan_policy="omit"
-            )
-        return norm, p
-
-    def is_constant(self, var_name):
-        """Determine if data by var_name is constant
-
-        Parameters
-        ----------
-        var_name : int, str
-            The var_name to check (or index of variable)
-
-        Returns
-        ----------
-        bool
-            True if all values of var_name are the same (i.e., no variation)
-        """
-        data = self.get_data_by_var_name(var_name)
-        return np.all(data == data[0])
+        """The normality and normality p-value of the input array"""
+        return scipy_stats.normaltest(self.y, nan_policy=self.nan_policy)
 
     @property
-    def constant_vars(self):
-        """Get a list of all constant variables"""
-        return [v for v in self.var_names if self.is_constant(v)]
-
-    @property
-    def constant_var_indices(self):
-        """Get a list of all constant variable indices"""
-        return [i for i, v in enumerate(self.var_names) if self.is_constant(v)]
-
-    @property
-    def non_const_data(self):
-        """Return self.data excluding any constant variables"""
-        return np.delete(self.data, self.constant_var_indices, axis=1)
-
-    def linear_reg(self, y, saved_reg=None):
-        """Initialize a MultiVariableRegression class object
-
-        Parameters
-        ----------
-        y : np.ndarray, list
-            Dependent data based on DVHAStats.data
-        saved_reg : MultiVariableRegression, optional
-            If supplied, predicted values (y-hat) will be calculated with
-            DVHAStats.data and the regression from saved_reg. This is useful
-            if testing a regression model on new data.
-
-        Returns
-        ----------
-        MultiVariableRegression
-            A MultiVariableRegression class object.
-        """
-        return MultiVariableRegression(self.data, y, saved_reg)
-
-    def univariate_control_charts(
-        self,
-        std=3,
-        ucl_limit=None,
-        lcl_limit=None,
-        box_cox=False,
-        box_cox_alpha=None,
-        box_cox_lmbda=None,
-        const_policy="propagate",
-    ):
-        """
-        Calculate control limits for a standard univariate Control Chart
-
-        Parameters
-        ----------
-        std : int, float, optional
-            Number of standard deviations used to calculate if a y-value is
-            out-of-control
-        ucl_limit : float, optional
-            Limit the upper control limit to this value
-        lcl_limit : float, optional
-            Limit the lower control limit to this value
-        box_cox : bool, optional
-            Set to true to perform a Box-Cox transformation on data prior to
-            calculating the control chart statistics
-        box_cox_alpha : float, optional
-            If alpha is not None, return the 100 * (1-alpha)% confidence
-            interval for lmbda as the third output argument. Must be between
-            0.0 and 1.0.
-        box_cox_lmbda : float, optional
-            If lmbda is not None, do the transformation for that value.
-            If lmbda is None, find the lambda that maximizes the log-likelihood
-            function and return it as the second output argument.
-        const_policy : str
-            {‘propagate’, ‘raise’}
-            Defines how to handle when data is constant. The following
-            options are available (default is ‘propagate’):
-            ‘propagate’: returns nan
-            ‘raise’: throws an error
-
-        Returns
-        ----------
-        dict
-            ControlChartData class objects stored in a dictionary with
-            var_names and indices as keys (can use var_name or index)
-        """
-        kwargs = {"std": std, "ucl_limit": ucl_limit, "lcl_limit": lcl_limit}
-        data = {}
-        if box_cox:
-            if self.box_cox_data is None:
-                self.box_cox(
-                    alpha=box_cox_alpha,
-                    lmbda=box_cox_lmbda,
-                    const_policy=const_policy,
-                )
-            cc_data = self.box_cox_data
-            plot_title = "Univariate Control Chart with Box-Cox Transformation"
-        else:
-            cc_data = self.data
-            plot_title = None
-        for i, key in enumerate(self.var_names):
-            if const_policy == "propagate" and np.all(np.isnan(cc_data[:, i])):
-                plot_title = "Cannot calculate control chart with const data!"
-            data[key] = ControlChartData(
-                cc_data[:, i], var_name=key, plot_title=plot_title, **kwargs
-            )
-            data[i] = data[key]
-        return data
-
-    def hotelling_t2(
-        self,
-        alpha=0.05,
-        box_cox=False,
-        box_cox_alpha=None,
-        box_cox_lmbda=None,
-        const_policy="omit",
-    ):
-        """
-        Calculate control limits for a standard univariate Control Chart
-
-        Parameters
-        ----------
-        alpha : float
-            Significance level used to determine the upper control limit (ucl)
-        box_cox : bool, optional
-            Set to true to perform a Box-Cox transformation on data prior to
-            calculating the control chart statistics
-        box_cox_alpha : float, optional
-            If alpha is not None, return the 100 * (1-alpha)% confidence
-            interval for lmbda as the third output argument. Must be between
-            0.0 and 1.0.
-        box_cox_lmbda : float, optional
-            If lmbda is not None, do the transformation for that value.
-            If lmbda is None, find the lambda that maximizes the log-likelihood
-            function and return it as the second output argument.
-        const_policy : str
-            {‘raise’, 'omit'}
-            Defines how to handle when data is constant. The following
-            options are available (default is ‘raise’):
-            ‘raise’: throws an error
-            'omit': exclude constant variables from calculation
-
-        Returns
-        ----------
-        HotellingT2
-            HotellingT2 class object
-        """
-
-        if box_cox:
-            if self.box_cox_data is None:
-                p = "propagate" if const_policy == "omit" else const_policy
-                self.box_cox(
-                    alpha=box_cox_alpha, lmbda=box_cox_lmbda, const_policy=p
-                )
-            data = self.box_cox_data
-            if const_policy == "omit":
-                data = data[:, ~np.all(np.isnan(data), axis=0)]
-            plot_title = (
-                "Multivariate Control Chart with Box-Cox Transformation"
-            )
-        else:
-            data = self.non_const_data if const_policy == "omit" else self.data
-            plot_title = None
-
-        return HotellingT2(data, alpha, plot_title=plot_title)
-
-    def box_cox_by_index(
-        self, index, alpha=None, lmbda=None, const_policy="propagate"
-    ):
-        """
-
-        Parameters
-        ----------
-        index : int, str
-            The index corresponding to the variable data to have a box-cox
-            transformation applied.  If index is a string, it will be assumed
-            to be the var_name
-        lmbda : None, scalar, optional
-            If lmbda is not None, do the transformation for that value.
-            If lmbda is None, find the lambda that maximizes the
-            log-likelihood function and return it as the second output
-            argument.
-        alpha : None, float, optional
-            If alpha is not None, return the 100 * (1-alpha)% confidence
-            interval for lmbda as the third output argument. Must be between
-            0.0 and 1.0.
-        const_policy : str
-            {‘propagate’, ‘raise’}
-            Defines how to handle when data is constant. The following
-            options are available (default is ‘propagate’):
-            ‘propagate’: returns nan
-            ‘raise’: throws an error
-        """
-        if self.box_cox_data is None:
-            self.box_cox_data = np.zeros_like(self.data)
-
-        if isinstance(index, str):
-            index = self.get_index_by_var_name(index)
-
-        if const_policy == "omit":
-            pass
-
-        try:
-            result, _ = scipy_stats.boxcox(
-                self.data[:, index], alpha=alpha, lmbda=lmbda
-            )
-        except ValueError as e:
-            if const_policy == "propagate":
-                self.box_cox_data[:, index] = np.array(
-                    [np.nan] * self.observations
-                )
-                return np.nan
-            raise e
-
-        self.box_cox_data[:, index] = result
-
-        return self.box_cox_data[:, index]
-
-    def box_cox(self, alpha=None, lmbda=None, const_policy="propagate"):
-        """Apply box_cox_by_index for all data"""
-        for i in range(self.variable_count):
-            self.box_cox_by_index(
-                i, alpha=alpha, lmbda=lmbda, const_policy=const_policy
-            )
-
-    def add_tend_line(self, var_name, plot_index):
-        """Add trend line based on moving average"""
-        trend_x, trend_y = moving_avg(
-            self.get_data_by_var_name(var_name), self.avg_len
-        )
-        self.plots[plot_index].add_line(
-            trend_y, x=trend_x, line_color="black", line_width=0.75
-        )
-
-    def pca(self, n_components=0.95, transform=True, **kwargs):
-        """Return an sklearn PCA-like object, see PCA object for details"""
-        return PCA(
-            self.data,
-            self.var_names,
-            n_components=n_components,
-            transform=transform,
-            **kwargs
-        )
-
-    def show(self, var_name):
-        """Display a plot of var_name with matplotlib"""
-        index = self.get_index_by_var_name(var_name)
-        var_name = self.var_names[index]
-        self.plots.append(
-            plot.Plot(
-                self.data[:, index],
-                x=self.x_axis,
-                xlabel="Observation",
-                ylabel=var_name,
-                title="",
-            )
-        )
-        self.add_tend_line(var_name, -1)
-        return self.plots[-1].figure.number
+    def chart_data(self):
+        """JSON compatible dict for chart generation"""
+        hist, bins = np.histogram(self.y, bins=self.bins)
+        center = (bins[:-1] + bins[1:]) / 2.0
+        norm, norm_p = self.normality
+        return {
+            "x": center.tolist(),
+            "y": hist.tolist(),
+            "mean": float(self.mean),
+            "median": float(self.median),
+            "std": float(self.std),
+            "normality": float(norm),
+            "normality_p": float(norm_p),
+        }
 
 
 class MultiVariableRegression:
@@ -453,15 +84,15 @@ class MultiVariableRegression:
 
         Parameters
         ----------
-        X : np.ndarray
+        X : array-like
             Independent data
-        y : np.ndarray, list
+        y : array-like
             Dependent data
         saved_reg : MultiVariableRegression, optional
             Optionally provide a previously calculated regression
         """
 
-        self.X = X
+        self.X = np.array(X) if isinstance(X, list) else X
         self.y = np.array(y) if isinstance(y, list) else y
 
         self._do_fit(saved_reg=saved_reg)
@@ -554,19 +185,20 @@ class MultiVariableRegression:
 
     @property
     def f_p_value(self):
+        """p-value of the f-statistic"""
         return scipy_stats.f.cdf(self.f_stat, self.df_model, self.df_error)
 
 
-class ControlChartData(DVHAStatsBaseClass):
+class ControlChart:
+    """Univariate Control Chart"""
+
     def __init__(
         self,
         y,
         std=3,
         ucl_limit=None,
         lcl_limit=None,
-        var_name=None,
         x=None,
-        plot_title=None,
     ):
         """
         Calculate control limits for a standard univariate Control Chart
@@ -582,26 +214,17 @@ class ControlChartData(DVHAStatsBaseClass):
             Limit the upper control limit to this value
         lcl_limit : float, optional
             Limit the lower control limit to this value
-        plot_title : str, optional
-            Over-ride the plot title
         """
-        DVHAStatsBaseClass.__init__(self)
 
         self.y = np.array(y) if isinstance(y, list) else y
+        self.x = x if x else np.linspace(1, len(self.y), len(self.y))
         self.std = std
         self.ucl_limit = ucl_limit
         self.lcl_limit = lcl_limit
-        self.var_name = var_name
-        self.x = x
-        self.plot_title = (
-            "Univariate Control Chart" if plot_title is None else plot_title
-        )
 
         # since moving range is calculated based on 2 consecutive points
         self.scalar_d = 1.128
 
-        self.plots = []
-
     def __str__(self):
         """String representation of ControlChartData object"""
         msg = [
@@ -615,26 +238,18 @@ class ControlChartData(DVHAStatsBaseClass):
         """Return the string representation"""
         return str(self)
 
-    @property
-    def y_no_nan(self):
-        """Remove indices with values of np.nan"""
-        return self.y[~np.isnan(self.y)]
-
     @property
     def center_line(self):
-        """Center line"""
-        data = self.y_no_nan
+        """Center line of charting data (i.e., mean value)"""
+        data = remove_nan(self.y)
         if len(data):
-            return np.mean(self.y_no_nan)
+            return np.mean(data)
         return np.nan
 
     @property
     def avg_moving_range(self):
         """Avg moving range based on 2 consecutive points"""
-        data = self.y_no_nan
-        if len(data):
-            return np.mean(np.absolute(np.diff(self.y_no_nan)))
-        return np.nan
+        return avg_moving_range(self.y, nan_policy="omit")
 
     @property
     def sigma(self):
@@ -690,28 +305,24 @@ class ControlChartData(DVHAStatsBaseClass):
         lcl, _ = self.control_limits
         return np.argwhere(self.y < lcl)
 
-    def show(self):
-        """Display the univariate control chart with matplotlib"""
+    @property
+    def chart_data(self):
+        """JSON compatible dict for chart generation"""
         lcl, ucl = self.control_limits
-        self.plots.append(
-            plot.ControlChart(
-                self.y,
-                self.out_of_control,
-                self.center_line,
-                x=self.x,
-                title=self.plot_title,
-                lcl=lcl,
-                ucl=ucl,
-                ylabel=self.var_name,
-            )
-        )
-        return self.plots[-1].figure.number
+        return {
+            "x": self.x.tolist(),
+            "y": self.y.tolist(),
+            "out_of_control": self.out_of_control.tolist(),
+            "center_line": float(self.center_line),
+            "lcl": float(lcl),
+            "ucl": float(ucl),
+        }
 
 
-class HotellingT2(DVHAStatsBaseClass):
+class HotellingT2:
     """Hotelling's t-squared statistic for multivariate hypothesis testing"""
 
-    def __init__(self, data, alpha=0.05, plot_title=None):
+    def __init__(self, data, alpha=0.05, const_policy="raise"):
         """Initialize the Hotelling T^2 class
 
         Parameters
@@ -722,18 +333,19 @@ class HotellingT2(DVHAStatsBaseClass):
         alpha : float
             The significance level used to calculate the
             upper control limit (UCL)
-        plot_title : str, optional
-            Over-ride the plot title
+        const_policy : str
+            {‘raise’, 'omit'}
+            Defines how to handle when data is constant. The following
+            options are available (default is ‘raise’):
+            ‘raise’: throws an error
+            'omit': exclude constant variables from calculation
         """
-        DVHAStatsBaseClass.__init__(self)
 
-        self.data = data
+        self.data = (
+            data if const_policy == "raise" else remove_const_column(data)
+        )
         self.alpha = alpha
         self.lcl = 0
-        self.plot_title = (
-            "Multivariate Control Chart" if plot_title is None else plot_title
-        )
-        self.plots = []
 
     def __str__(self):
         """String representation of HotellingT2 object"""
@@ -751,17 +363,17 @@ class HotellingT2(DVHAStatsBaseClass):
 
     @property
     def observations(self):
-        """Number of observations"""
+        """Number of observations in data"""
         return self.data.shape[0]
 
     @property
     def variable_count(self):
-        """Number of variables"""
+        """Number of variables in data"""
         return self.data.shape[1]
 
     @property
     def Q(self):
-        """Calculate Hotelling T^2 from a 2-D numpy array
+        """Calculate Hotelling T^2 statistic (Q) from a 2-D numpy array
 
         Returns
         -------
@@ -780,7 +392,7 @@ class HotellingT2(DVHAStatsBaseClass):
 
     @property
     def center_line(self):
-        """Center line"""
+        """Center line (median value of beta distribution)"""
         return self.get_control_limit(0.5)
 
     @property
@@ -815,25 +427,22 @@ class HotellingT2(DVHAStatsBaseClass):
         N = self.observations
         a = self.variable_count / 2
         b = (N - self.variable_count - 1) / 2
-        return ((N - 1) ** 2 / N) * beta.ppf(x, a, b)
+        return ((N - 1) ** 2 / N) * scipy_stats.beta.ppf(x, a, b)
 
-    def show(self):
-        """Display the multivariate control chart with matplotlib"""
-        self.plots.append(
-            plot.ControlChart(
-                self.Q,
-                self.out_of_control,
-                self.center_line,
-                title=self.plot_title,
-                ucl=self.ucl,
-                lcl=self.lcl,
-                ylabel="Hottelling T^2",
-            )
-        )
-        return self.plots[-1].figure.number
+    @property
+    def chart_data(self):
+        """JSON compatible dict for chart generation"""
+        return {
+            "x": list(range(1, self.observations + 1)),
+            "y": self.Q.tolist(),
+            "out_of_control": self.out_of_control.tolist(),
+            "center_line": float(self.center_line),
+            "lcl": float(self.lcl),
+            "ucl": float(self.ucl),
+        }
 
 
-class PCA(sklearnPCA, DVHAStatsBaseClass):
+class PCA(sklearn_PCA):
     """Hotelling's t-squared statistic for multivariate hypothesis testing"""
 
     def __init__(
@@ -843,38 +452,39 @@ class PCA(sklearnPCA, DVHAStatsBaseClass):
 
         Parameters
         ----------
-        X : array-like, shape (n_samples, n_features)
-            Training data, where n_samples is the number of samples and
-            n_features is the number of features.
-        var_names : str, optional
-            Names of the independent variables in X
+        X : np.ndarray
+            Training data (2-D), where n_samples is the number of samples and
+            n_features is the number of features. shape (n_samples, n_features)
+        var_names : list, optional
+            Optionally provide names of the features
         n_components : int, float, None or str
             Number of components to keep. if n_components is not set all
             components are kept:
-                n_components == min(n_samples, n_features)
+            n_components == min(n_samples, n_features)
 
-                If n_components == 'mle' and svd_solver == 'full', Minka’s MLE
-                is used to guess the dimension. Use of n_components == 'mle'
-                will interpret svd_solver == 'auto' as svd_solver == 'full'.
+            If n_components == 'mle' and svd_solver == 'full', Minka’s MLE
+            is used to guess the dimension. Use of n_components == 'mle'
+            will interpret svd_solver == 'auto' as svd_solver == 'full'.
 
-                If 0 < n_components < 1 and svd_solver == 'full', select the
-                number of components such that the amount of variance that
-                needs to be explained is greater than the percentage specified
-                by n_components.
+            If 0 < n_components < 1 and svd_solver == 'full', select the
+            number of components such that the amount of variance that
+            needs to be explained is greater than the percentage specified
+            by n_components.
 
-                If svd_solver == 'arpack', the number of components must be
-                strictly less than the minimum of n_features and n_samples.
+            If svd_solver == 'arpack', the number of components must be
+            strictly less than the minimum of n_features and n_samples.
         transform : bool
             Fit the model and apply the dimensionality reduction
         kwargs : any
             Provide any keyword arguments for sklearn.decomposition.PCA:
             https://scikit-learn.org/stable/modules/generated/sklearn.decomposition.PCA.html
         """
-        DVHAStatsBaseClass.__init__(self)
+
         self.X = X
-        self.var_names = range(X.shape[1]) if var_names is None else var_names
-        self.plots = []
-        sklearnPCA.__init__(self, n_components=n_components, **kwargs)
+        self.var_names = (
+            list(range(self.X.shape[1])) if var_names is None else var_names
+        )
+        sklearn_PCA.__init__(self, n_components=n_components, **kwargs)
 
         if transform:
             self.fit_transform(self.X)
@@ -885,31 +495,19 @@ class PCA(sklearnPCA, DVHAStatsBaseClass):
     def feature_map_data(self):
         return self.components_
 
-    def show(self, plot_type="feature_map", absolute=True):
-        """Create a heat map of PCA components
-
-        Parameters
-        ----------
-        plot_type : str
-            Select a plot type to display. Options include: feature_map.
-        absolute : bool
-            Heat map will display the absolute values in PCA components
-            if True
-        """
-        if plot_type == "feature_map":
-            data = self.feature_map_data
-            if absolute:
-                data = abs(data)
-            self.plots.append(plot.PCAFeatureMap(data, self.var_names))
-            return self.plots[-1].figure.number
+    @property
+    def component_labels(self):
+        """Get component names"""
+        return [
+            "%s Comp" % (get_ordinal(n + 1))
+            for n in range(self.components_.shape[0])
+        ]
 
 
-class CorrelationMatrix(DVHAStatsBaseClass):
+class CorrelationMatrix:
     """Pearson-R correlation matrix"""
 
-    def __init__(
-        self, X, var_names=None, corr_type="Pearson", cmap="coolwarm"
-    ):
+    def __init__(self, X, corr_type="Pearson"):
         """Initialization of CorrelationMatrix object
 
         Parameters
@@ -919,14 +517,9 @@ class CorrelationMatrix(DVHAStatsBaseClass):
             p columns of variables.
         corr_type : str
             Either "Pearson" or "Spearman"
-        cmap : str
-            matplotlib compatible color map
         """
-        DVHAStatsBaseClass.__init__(self)
         self.X = X
-        self.var_names = range(X.shape[1]) if var_names is None else var_names
         self.corr_type = corr_type.lower()
-        self.cmap = cmap
 
         if self.corr_type not in {"pearson", "spearman"}:
             msg = "Invalid corr_type: must be either 'Pearson' or 'Spearman'"
@@ -939,42 +532,25 @@ class CorrelationMatrix(DVHAStatsBaseClass):
         if self.corr_type in list(func_map):
             self.corr, self.p = func_map[self.corr_type](self.X)
 
-    def __title(self, is_corr=True):
-        """Get plot title
+    @property
+    def normality(self):
+        return normality(self.X, nan_policy="omit")
 
-        Parameters
-        ----------
-        is_corr : bool
-            Set to True if plot data type is correlation, False if p-value
-        """
-        mat_type = "Pearson-R" if self.corr_type == "pearson" else "Spearman"
-        value_type = ["p-value", "Correlation"][is_corr]
-        return "%s %s Matrix" % (mat_type, value_type)
+    @property
+    def chart_data(self):
+        """JSON compatible dict for chart generation"""
+        norm, norm_p = self.normality
+        return {
+            "corr": self.corr.tolist(),
+            "p": self.p.tolist(),
+            "norm": norm.tolist(),
+            "norm_p": norm_p.tolist(),
+        }
 
-    def show(self, absolute=False, corr=True):
-        """Create a heat map of PCA components
 
-        Parameters
-        ----------
-        absolute : bool
-            Heat map will display the absolute values in PCA components
-            if True
-        corr : bool
-            Plot a p-value matrix if False, correlation matrix if True.
-        """
-
-        data = self.corr if corr else self.p
-        data = abs(data) if absolute else data
-        self.plots.append(
-            plot.HeatMap(
-                data,
-                xlabels=self.var_names,
-                ylabels=self.var_names,
-                cmap=self.cmap,
-                title=self.__title(corr),
-            )
-        )
-        return self.plots[-1].figure.number
+#########################################################
+# Pure stats functions
+#########################################################
 
 
 def get_lin_reg_p_values(X, y, predictions, y_intercept, slope):
@@ -1075,3 +651,253 @@ def spearman_correlation_matrix(X, nan_policy="omit"):
         ‘omit’: performs the calculations ignoring nan values
     """
     return scipy_stats.spearmanr(X, nan_policy=nan_policy)
+
+
+def moving_avg(y, avg_len, x=None, weight=None):
+    """Calculate the moving (rolling) average of a set of data
+
+    Parameters
+    ----------
+    y : array-like
+        data (1-D) to be averaged
+    avg_len : int
+        Data is averaged over this many points (current value and avg_len - 1
+        prior points)
+    x : np.ndarray, list, optional
+        Optionally specify the x-axis values. Otherwise index+1 is used.
+    weight : np.ndarray, list, optional
+        A weighted moving average is calculated based on the provided weights.
+        weight must be of same length as y. Weights of one are assumed by
+        default.
+
+    Returns
+    ----------
+    array-like
+        x-values, moving average values
+    """
+    x = np.linspace(1, len(y), len(y)) if x is None else x
+    weight = np.ones_like(y) if weight is None else weight
+
+    cumsum, moving_aves, x_final = [0], [], []
+
+    for i, yi in enumerate(y, 1):
+        cumsum.append(cumsum[i - 1] + yi / weight[i - 1])
+        if i >= avg_len:
+            moving_ave = (cumsum[i] - cumsum[i - avg_len]) / avg_len
+            moving_aves.append(moving_ave)
+    x_final = [x[i] for i in range(avg_len - 1, len(x))]
+
+    return np.array(x_final), np.array(moving_aves)
+
+
+def normality(data, nan_policy="omit"):
+    """Calculate the normality and associated p-values for each variable
+
+    Parameters
+    ----------
+    data : np.ndarray
+        A 2-D array of data
+    nan_policy : str
+        Value must be one of the following: {‘propagate’, ‘raise’, ‘omit’}
+        Defines how to handle when input contains nan. The following options
+        are available (default is ‘omit’):
+        ‘propagate’: returns nan
+        ‘raise’: throws an error
+        ‘omit’: performs the calculations ignoring nan values
+
+    Returns
+    ----------
+    np.ndarray, np.ndarray
+        A tuple of 1-D arrays are returned: Normality and its
+        p-values.
+    """
+    var_count = data.shape[1]
+    norm, p = np.zeros(var_count), np.zeros(var_count)
+    for i in range(var_count):
+        norm[i], p[i] = scipy_stats.normaltest(
+            data[:, i], nan_policy=nan_policy
+        )
+    return norm, p
+
+
+def box_cox(arr, alpha=None, lmbda=None, const_policy="propagate"):
+    """
+
+    Parameters
+    ----------
+    arr : np.ndarray
+        Input array. Must be positive 1-dimensional.
+    lmbda : None, scalar, optional
+        If lmbda is not None, do the transformation for that value.
+        If lmbda is None, find the lambda that maximizes the
+        log-likelihood function and return it as the second output
+        argument.
+    alpha : None, float, optional
+        If alpha is not None, return the 100 * (1-alpha)% confidence
+        interval for lmbda as the third output argument. Must be between
+        0.0 and 1.0.
+    const_policy : str
+        {‘propagate’, ‘raise’, 'omit'}
+        Defines how to handle when data is constant. The following
+        options are available (default is ‘propagate’):
+        ‘propagate’: returns nan
+        ‘raise’: throws an error
+    """
+
+    try:
+        box_cox_data, _ = scipy_stats.boxcox(arr, alpha=alpha, lmbda=lmbda)
+    except ValueError as e:
+        if const_policy == "propagate":
+            return np.array([np.nan] * arr.shape[0])
+        raise e
+
+    return box_cox_data
+
+
+def avg_moving_range(arr, nan_policy="omit"):
+    """Calculate the average moving range (over 2-consecutive point1)
+
+    Parameters
+    ----------
+    arr : array-like (1-D)
+        Input array. Must be positive 1-dimensional.
+    nan_policy : str, optional
+        Value must be one of the following: {‘propagate’, ‘raise’, ‘omit’}
+        Defines how to handle when input contains nan. The following options
+        are available (default is ‘omit’):
+        ‘propagate’: returns nan
+        ‘raise’: throws an error
+        ‘omit’: performs the calculations ignoring nan values
+
+    Returns
+    ----------
+    np.ndarray, np.nan
+        Average moving range. Returns NaN if arr is empty
+    """
+
+    arr = process_nan_policy(arr, nan_policy)
+    if len(arr) == 0:
+        return np.nan
+    return np.mean(np.absolute(np.diff(arr)))
+
+
+###################################################
+# Stats related utilities
+###################################################
+
+
+def remove_nan(arr):
+    """Remove indices from 1-D array with values of np.nan
+
+    Parameters
+    ----------
+    arr : np.ndarray (1-D)
+        Input array. Must be positive 1-dimensional.
+
+    Returns
+    ----------
+    np.ndarray
+        arr with NaN values deleted
+
+    """
+    return arr[~np.isnan(arr)]
+
+
+def remove_const_column(arr):
+    """Remove all columns with zero variance
+
+    Parameters
+    ----------
+    arr : np.ndarray
+        Input array (2-D)
+
+    Returns
+    ----------
+    np.ndarray
+        Input array with columns of a constant value removed
+    """
+    return arr[:, ~np.all(np.isnan(arr), axis=0)]
+
+
+def is_nan_arr(arr):
+    """Check if array has only NaN elements
+
+    Parameters
+    ----------
+    arr : np.ndarray
+        Input array
+
+    Returns
+    ----------
+    bool
+        True if all elements are np.nan
+    """
+    return np.all(np.isnan(arr))
+
+
+def is_arr_constant(arr):
+    """Determine if data by var_name is constant
+
+    Parameters
+    ----------
+    arr : array-like
+        Input array or object that can be converted to an array
+
+    Returns
+    ----------
+    bool
+        True if all values the same (i.e., no variation)
+    """
+    return np.all(arr == arr[0])
+
+
+def process_nan_policy(arr, nan_policy):
+    """Calculate the average moving range (over 2-consecutive point1)
+
+    Parameters
+    ----------
+    arr : array-like (1-D)
+        Input array. Must be positive 1-dimensional.
+    nan_policy : str
+        Value must be one of the following: {‘propagate’, ‘raise’, ‘omit’}
+        Defines how to handle when input contains nan. The following options
+        are available (default is ‘omit’):
+        ‘propagate’: returns nan
+        ‘raise’: throws an error
+        ‘omit’: performs the calculations ignoring nan values
+
+    Returns
+    ----------
+    np.ndarray, np.nan
+        Input array evaluated per nan_policy
+    """
+
+    arr_no_nan = remove_nan(arr)
+    if len(arr_no_nan) != len(arr):
+        if nan_policy == "raise":
+            msg = "NaN values are not supported for avg_moving_range"
+            raise NotImplementedError(msg)
+        if nan_policy == "propagate":
+            return np.nan
+        if nan_policy == "omit":
+            return arr_no_nan
+    return arr
+
+
+def get_ordinal(n):
+    """Convert number to its ordinal (e.g., 1 to 1st)
+
+    Parameters
+    ----------
+    n : int
+        Number to be converted to ordinal
+
+    Returns
+    ----------
+    str
+        the ordinal of n
+    """
+    return "%d%s" % (
+        n,
+        "tsnrhtdd"[(n // 10 % 10 != 1) * (n % 10 < 4) * n % 10 :: 4],
+    )
